# vim:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:tw=0

  #############################################################################
  #
  # Copyright (c) 2005 Dell Computer Corporation
  # Dual Licenced under GNU GPL and OSL
  #
  #############################################################################
"""
repository module
"""

from __future__ import generators

import sys

import package
from firmwaretools.trace_decorator import decorate, traceLog, getLog

class DepParser(object):
    tokens = ( 'ID', 'LT', 'LE', 'EQ', 'GE', 'GT', 'COMMA' )
    t_ID   = r'[\w()]+'
    t_LT   = r'<'
    t_LE   = r'<='
    t_EQ   = r'=='
    t_GE   = r'>='
    t_GT   = r'>'
    t_COMMA = r','
    t_ignore = " \t"

    def t_error(self, t):
        print "Illegal character '%s'" % t.value[0]
        t.skip(1)

    decorate(traceLog())
    def __init__(self, string, inventory, fullInventory, *args, **kargs):
        self.inventory = inventory
        self.fullInventory = fullInventory
        self.depPass = 1

        import ply_lex
        lexer = ply_lex.lex( module=self )

        import ply_yacc
        parser = ply_yacc.yacc( module=self, write_tables=0, debug=0 )

        parser.parse(string, lexer=lexer, debug=0)

    precedence = (
        ('left', 'COMMA'),
        )

    def p_error(self, t):
        print "Syntax error at '%s'" % t

    def p_stmt(self, t):
        # statement_list can be 1) empty, 2) single statement, or 3) list
        """statement_list :
                          | statement
                          | statement_list COMMA statement
           statement : dep"""
        pass

    def p_package_depencency(self, t):
        """dep : ID LT ID
               | ID LE ID
               | ID EQ ID
               | ID GE ID
               | ID GT ID
        """
        op  = t[2]
        reqPkg = package.Package (name=t[1], version=t[3], displayname="virtual package")
        pkg = self.inventory.get(t[1])
        if pkg:
            r = pkg.compareVersion(reqPkg)
            evalStr = "%s %s 0" % (r, op)
            if not eval(evalStr):
                self.reason = "Failed for rule: requires %s %s %s" % (t[1], t[2], t[3])
                self.depPass = 0
        else:
            self.reason = "Repository package doesn't exist in system inventory."
            self.depPass = 0


    def p_package_exists(self, t):
        """dep : ID"""
        if not self.inventory.get(t[1]):
            self.reason = "Failed for rule: requires %s" % t[1]
            self.depPass = 0



# vim:ai:ts=4:sw=4:et:filetype=python:

#future imports always first
from __future__ import generators

# std python stuff
import logging
import gtk

from firmwaretools.pycompat import runLongProcess
from firmwaretools.trace_decorator import decorate, traceLog, getLog

moduleLog = getLog()
moduleVerboseLog = getLog(prefix="verbose.")


decorate(traceLog())
def getSelectionPaths(treeview):
    def func(model, path, iterator, data):
        model = None
        iterator = None
        data.append(path)

    paths = []
    treeselection = treeview.get_selection()
    treeselection.selected_foreach(func, paths)
    return paths

decorate(traceLog())
def gtkYield():
    # process gui events during long-running loops
    # so that we are more responsive
    while gtk.events_pending():
        gtk.main_iteration(False)


decorate(traceLog())
def runLongProcessGtk(function, args=None, kargs=None, waitLoopFunction=None):
    decorate(traceLog())
    def myFunc():
        # can access outer function variables
        if waitLoopFunction is not None:
            waitLoopFunction()
        gtkYield() # make sure current GUI is fully displayed

    return runLongProcess(function, args, kargs, waitLoopFunction=myFunc)



# vim:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:tw=0

  #############################################################################
  #
  # Copyright (c) 2005 Dell Computer Corporation
  # Dual Licenced under GNU GPL and OSL
  #
  #############################################################################
"""module

some docs here eventually.
"""

from __future__ import generators

import os
import time
import logging

# import arranged alphabetically
import package
from firmwaretools.trace_decorator import decorate, traceLog, getLog
import firmwaretools.plugins as plugins

plugin_type = (plugins.TYPE_MOCK_INVENTORY,)
requires_api_version = "2.0"

moduleLog = getLog()
moduleVerboseLog = getLog(prefix="verbose.")

#==============================================================
# mock classes for unit tests
#   plus expected data returns
#==============================================================
decorate(traceLog())
def inventory_hook(conduit, inventory=None, *args, **kargs):
    base = conduit.getBase()
    cb = base.cb

    import firmwaretools as ft
    ft.callCB(cb, who="mock_inventory", what="running_inventory", details="fake cmd")
    inventory.addDevice( package.Device(
            name = "debug_system_bios",
            displayname = "System BIOS for Imaginary Server 1234",
            version = "A02"))
    ft.callCB(cb, who="mock_inventory", what="running_inventory", details="fake cmd 2")
    inventory.addDevice( package.Device(
            name = "debug_system_bmc",
            displayname = "Baseboard Management Controller for Imaginary Server 1234",
            version = "1.0"))
    ft.callCB(cb, who="mock_inventory", what="running_inventory", details="fake cmd 3")
    inventory.addDevice( package.Device(
            name = "debug_pci_firmware_ven_crappy_dev_slow",
            displayname = "ReallyFast Network Controller",
            version = "1.0"))
    ft.callCB(cb, who="mock_inventory", what="running_inventory", details="fake cmd 4")
    inventory.addDevice( package.Device(
            name = "debug_pci_firmware_ven_0x0c64_dev_0xrocked",
            displayname = "Pokey Modem -- Enhanced 1200baud",
            version = "2.0"))
    ft.callCB(cb, who="mock_inventory", what="running_inventory", details="fake cmd 5")
    inventory.addDevice( package.Device(
            name = "debug_pci_firmware_ven_corrupt_dev_yourdata",
            displayname = "SafeData RAID Controller v2i",
            version = "2.0"))
    ft.callCB(cb, who="mock_inventory", what="running_inventory", details="fake cmd 6")
    inventory.addDevice( package.Device(
            name = "debug_pci_firmware_ven_violates_dev_scsistandard",
            displayname = "AdapFirm SloTek AHA-1501",
            version = "3.0"))
    ft.callCB(cb, who="mock_inventory", what="running_inventory", details="fake cmd 7")
    inventory.addDevice( package.Device(
            name = "debug_pci_firmware_ven_draws_dev_polygons",
            displayname = "PixelPusher 2000 Video Adapter",
            version = "4.0"))


#new style -- used by unit tests.
class MockPackage2(package.RepositoryPackage):
    decorate(traceLog())
    def __init__(self, *args, **kargs):
        super(MockPackage2, self).__init__(*args, **kargs)

    decorate(traceLog())
    def install(self):
        self.status = "in_progress"
        self.status = "success"
        return "SUCCESS"

# used when we switch to 'fake' data
class MockRepositoryPackage(package.RepositoryPackage):
    decorate(traceLog())
    def __init__(self, *args, **kargs):
        super(MockRepositoryPackage, self).__init__(*args, **kargs)
        self.capabilities['can_downgrade'] = True
        self.capabilities['can_reflash'] = True
        self.capabilities['accurate_update_percentage'] = True
        self.uniqueInstance = self.name

    decorate(traceLog())
    def install(self):
        self.status = "in_progress"
        for i in xrange(100):
            self.progressPct = i/100.0
            time.sleep(0.01)
        #print "MockRepositoryPackage -> Install pkg(%s)  version(%s)" % (str(self), self.version)
        self.progressPct = 1
        self.status = "success"







#==============================================================
# mock classes for unit tests
#   plus expected data returns
#==============================================================

mockExpectedOutput = """debug_pci_firmware_ven_0x0c64_dev_0xrocked
debug_pci_firmware_ven_crappy_dev_slow
debug_system_bmc
debug_pci_firmware_ven_corrupt_dev_yourdata
debug_system_bios
debug_pci_firmware_ven_draws_dev_polygons
debug_pci_firmware_ven_violates_dev_scsistandard"""


# re-use mock data from low-level getSystemId mock function
mockExpectedOutput_inventory = [("mock_package(ven_0x1028_dev_0x1234)", "a05"), ]

#==============================================================
# mock classes for unit tests
#   plus expected data returns
#==============================================================

# re-use mock data from low-level getSystemId mock function
mockExpectedOutput_bootstrap = """mock_package(ven_0x1028_dev_0x1234)"""




# vim:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:tw=0

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


"""
Firmware-tools: update infrastructure for firmware
"""

import ConfigParser
import fcntl
import glob
import logging
import logging.config
import os
import sys

from trace_decorator import decorate, traceLog, getLog
import errors
import repository

#import config
import plugins

def mkselfrelpath(*args):
    return os.path.normpath(os.path.join(os.path.dirname(os.path.realpath(__file__)), *args))

# these are replaced by autotools when installed.
__VERSION__="2.1.14"
SYSCONFDIR="/usr/local/etc"
PYTHONDIR="/usr/local/lib/python2.4/site-packages"
PKGPYTHONDIR="/usr/local/lib/python2.4/site-packages/firmwaretools"
PKGDATADIR="/usr/local/share/firmware-tools"
DATADIR="/usr/local/share"
PKGCONFDIR="/usr/local/etc/firmware/"
LOCALSTATEDIR=mkselfrelpath("..", "var")
# end build system subs

PID_FILE = '/var/run/ft.pid'

class confObj(object):
    def __getattribute__(self, name):
        return object.__getattribute__(self, name.lower())
    def __setattr__(self, name, value):
        object.__setattr__(self, name.lower(), value)

decorate(traceLog())
def callCB(cb, *args, **kargs):
    if cb is None: return
    try:
        return cb(*args, **kargs)
    except TypeError:
        pass

class Callback(object):
    def __init__(self):
        pass

    def __call__(self, *args, **kargs):
        func = getattr(self, kargs.get("what", "UNKNOWN"), None)
        if func is not None:
            return func(*args, **kargs)

class FtBase(object):
    """This is a primary structure and base class. It houses the objects and
       methods needed to perform most things . It is almost an abstract
       class in that you will need to add your own class above it for most
       real use."""

    def __init__(self):
        self.logger = getLog()
        self.verbose_logger = getLog(prefix="verbose.")

        self.cmdargs = []
        self.cb = None

        self._conf = None
        self._repo = None
        self._systemInventory = None
        self._vendorId = None
        self._systemId = None

        self.verbosity = 0
        self.trace = 0
        self.loggingConfig = os.path.join(PKGCONFDIR, "firmware.conf")

        # Start with plugins disabled
        self.disablePlugins()

    def _getConfig(self, cfgFiles=None, pluginTypes=(plugins.TYPE_CORE, plugins.TYPE_INVENTORY,), optparser=None, disabledPlugins=None):
        if self._conf is not None:
            return self._conf

        if cfgFiles is None:
            cfgFiles = [os.path.join(PKGCONFDIR, "firmware.conf"),]

        if disabledPlugins is None:
            disabledPlugins = []

        self.conf = confObj()

        self.setupLogging(self.loggingConfig, self.verbosity, self.trace)

        self.setConfFromIni(cfgFiles)

        self.conf.uid = os.geteuid()

        self.doPluginSetup(optparser, pluginTypes, disabledPlugins)

        return self._conf


    def setupLogging(self, configFile, verbosity=1, trace=0):
        # set up logging
        logging.config.fileConfig(configFile)
        root_log       = logging.getLogger()
        ft_log         = logging.getLogger("firmwaretools")
        ft_verbose_log = logging.getLogger("verbose")
        ft_trace_log   = logging.getLogger("trace")

        ft_log.propagate = 0
        ft_trace_log.propagate = 0
        ft_verbose_log.propagate = 0

        if verbosity >= 1:
            ft_log.propagate = 1
        if verbosity >= 2:
            ft_verbose_log.propagate = 1
        if verbosity >= 3:
            for hdlr in root_log.handlers:
                hdlr.setLevel(logging.DEBUG)
        if trace:
            ft_trace_log.propagate = 1

    decorate(traceLog())
    def setConfFromIni(self, cfgFiles):
        defaults = {
            "sysconfdir": SYSCONFDIR,
            "pythondir": PYTHONDIR,
            "datadir": DATADIR,
            "pkgpythondir": PKGPYTHONDIR,
            "pkgdatadir": PKGDATADIR,
            "pkgconfdir": PKGCONFDIR,
            "localstatedir": LOCALSTATEDIR,
        }
        self._ini = ConfigParser.SafeConfigParser(defaults)
        for i in cfgFiles:
            self._ini.read(i)

        mapping = {
            # conf.WHAT    : (iniSection, iniOption, default)
            "storageTopdir": ('main', 'storage_topdir', "%s/firmware" % DATADIR),
            "pluginSearchPath": ('main', 'plugin_search_path', os.path.join(PKGDATADIR, "plugins")),
            "pluginConfDir": ('main', 'plugin_config_dir', os.path.join(PKGCONFDIR, "firmware.d")),
            "rpmMode": ('main', 'rpm_mode', "manual"),
        }
        for key, val in mapping.items():
            if self._ini.has_option( val[0], val[1] ):
                setattr(self.conf, key, self._ini.get(val[0], val[1]))
            else:
                setattr(self.conf, key, val[2])

        # read plugin configs
        for i in glob.glob( "%s/*.conf" % self.conf.pluginConfDir ):
            self._ini.read(i)

    decorate(traceLog())
    def listPluginsFromIni(self):
        return [x[len("plugin:"):] for x in self._ini.sections() if x.startswith("plugin:")]

    decorate(traceLog())
    def getPluginConfFromIni(self, plugin):
        section = "plugin:%s" % plugin
        conf = confObj()

        conf.module = None
        conf.enabled = False
        conf.search = None

        for i in self._ini.options(section):
            setattr(conf, i, self._ini.get(section, i))

        #required ("enabled", "module"):
        if getattr(conf, "module", None) is None:
            conf.enabled = False

        return conf

    # called early so no tracing.
    def disablePlugins(self):
        '''Disable plugins
        '''
        self.plugins = plugins.DummyPlugins()

    decorate(traceLog())
    def doPluginSetup(self, optparser=None, pluginTypes=None, disabledPlugins=None):
        if isinstance(self.plugins, plugins.Plugins):
            raise RuntimeError("plugins already initialised")

        self.plugins = plugins.Plugins(self, optparser, pluginTypes, disabledPlugins)

    decorate(traceLog())
    def _getRepo(self):
        if self._repo is not None:
            return self._repo

        self._repo = repository.Repository( self.conf.storageTopdir )
        return self._repo

    decorate(traceLog())
    def _getInventory(self):
        if self._systemInventory is not None:
            return self._systemInventory

        self._systemInventory = repository.SystemInventory()
        self.plugins.run("preinventory", inventory=self._systemInventory)
        self.plugins.run("inventory", inventory=self._systemInventory)
        self.plugins.run("postinventory", inventory=self._systemInventory)
        return self._systemInventory

    decorate(traceLog())
    def calculateUpgradeList(self, cb=None):
        saveCb = self.cb
        self.cb = cb
        try:
            for candidate in self.repo.iterPackages(cb=cb):
                self.systemInventory.addAvailablePackage(candidate)

            self.systemInventory.calculateUpgradeList(cb)
        finally:
            self.cb = saveCb

        return self.systemInventory

    # properties so they auto-create themselves with defaults
    repo = property(fget=lambda self: self._getRepo(),
                     fset=lambda self, value: setattr(self, "_repo", value))
    conf = property(fget=lambda self: self._getConfig(),
                    fset=lambda self, value: setattr(self, "_conf", value),
                    fdel=lambda self: setattr(self, "_conf", None))
    systemInventory = property(
                    fget=lambda self: self._getInventory(),
                     fset=lambda self, value: setattr(self, "_systemInventory", value),
                    fdel=lambda self: setattr(self, "_systemInventory", None))

    decorate(traceLog())
    def lock(self):
        if self.conf.uid == 0:
            self.runLock = open(PID_FILE, "a+")
            try:
                fcntl.lockf(self.runLock.fileno(), fcntl.LOCK_EX | fcntl.LOCK_NB)
            except IOError, e:
                raise errors.LockError, "unable to obtain exclusive lock."

    decorate(traceLog())
    def unlock(self):
        if self.conf.uid == 0:
            fcntl.lockf(self.runLock.fileno(), fcntl.LOCK_UN)
            os.unlink(PID_FILE)

    decorate(traceLog())
    def setSystemId(self, vendorId, systemId):
        if not (vendorId and systemId):
            raise RuntimeError("Need non-null, non-zero, id for vendor and system id.")
        self._vendorId = vendorId
        self._systemId = systemId

    decorate(traceLog())
    def getSystemId(self):
        return (self._vendorId, self._systemId)

    decorate(traceLog())
    def yieldInventory(self, cb=None):
        saveCb = self.cb
        try:
            self.cb = cb
            for dev in self.systemInventory.iterDevices():
                yield dev
        except:
            self.cb = saveCb
            raise





# vim:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:tw=0

  #############################################################################
  #
  # Copyright (c) 2005 Dell Computer Corporation
  # Dual Licenced under GNU GPL and OSL
  #
  #############################################################################
"""module

some docs here eventually.
"""

from __future__ import generators

# import arranged alphabetically
import os
try:
    import subprocess
except ImportError:
    import compat_subprocess as subprocess

# my stuff
import firmwaretools.package as package
from firmwaretools.trace_decorator import decorate, traceLog, getLog
import firmwaretools.plugins as plugins

plugin_type = (plugins.TYPE_INVENTORY,)
requires_api_version = "2.0"

# ======
# public API
# ======

sysfs_pcidevdir="/sys/bus/pci/devices"

decorate(traceLog())
def inventory_hook(conduit, inventory=None, *args, **kargs):
    base = conduit.getBase()
    cb = base.cb
    devdir = sysfs_pcidevdir

    for d in os.listdir(devdir):
        d = makePciDevice(os.path.join(devdir, d))
        if inventory.getDevice(d.uniqueInstance) is None:
            inventory.addDevice(d)

decorate(traceLog())
def getFile(f):
    fd = open(f,"r")
    ret = fd.read()
    fd.close()
    if ret[-1:] == '\n': ret = ret[:-1]
    return ret

decorate(traceLog())
def chomp(s):
    if s.endswith("\n"):
        return s[:-1]
    return s

LSPCI = None
for i in ("/sbin/lspci", "/usr/bin/lspci"):
    if os.path.exists(i):
        LSPCI=i
        break


decorate(traceLog())
def makePciDevice(devDir):
    kargs = {}
    kargs["pciVendor"] = int(getFile(os.path.join(devDir, "vendor")),16)
    kargs["pciDevice"] = int(getFile(os.path.join(devDir, "device")),16)
    kargs["pciSubVendor"] = int(getFile(os.path.join(devDir, "subsystem_vendor")),16)
    kargs["pciSubDevice"] = int(getFile(os.path.join(devDir, "subsystem_device")),16)
    kargs["pciClass"] = int(getFile(os.path.join(devDir, "class")),16)

    shortname = None
    name = "pci_firmware(ven_0x%04x_dev_0x%04x" % (kargs["pciVendor"], kargs["pciDevice"])
    if kargs["pciSubVendor"] and kargs["pciSubDevice"]:
        shortname = name + ")"
        name = name + "_subven_0x%04x_subdev_0x%04x" % (kargs["pciSubVendor"], kargs["pciSubDevice"])
    name = name + ")"

    dirname = os.path.basename(devDir)
    dets = dirname.split(":")
    kargs["pciBDF_Domain"] = int(dets[0],16)
    kargs["pciBDF_Bus"] = int(dets[1],16)
    kargs["pciBDF_Device"] = int(dets[2].split(".")[0],16)
    kargs["pciBDF_Function"] = int(dets[2].split(".")[1],16)

    kargs["pciDbdf"] = (kargs["pciBDF_Domain"], kargs["pciBDF_Bus"], kargs["pciBDF_Device"], kargs["pciBDF_Function"])

    null = open("/dev/null", "w")
    p = subprocess.Popen([LSPCI, "-s", "%02x:%02x:%02x.%x" % kargs["pciDbdf"]], stdout=subprocess.PIPE, stderr=null, stdin=null)
    lspciname = chomp(p.communicate()[0])
    null.close()

    if lspciname is not None and len(lspciname) > 0:
        displayname = lspciname
    else:
        displayname = "unknown device"

    if shortname is not None:
        return package.PciDevice(
            name=name,
            shortname=shortname,
            version='unknown',
            displayname=displayname,
            lspciname=lspciname,
            **kargs
            )
    else:
        return package.PciDevice(
            name=name,
            version='unknown',
            displayname=displayname,
            lspciname=lspciname,
            **kargs
            )

if __name__ == "__main__":
    for p in getPciDevs():
        print "%s" % p.name




# vim:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:tw=0

  #############################################################################
  #
  # Copyright (c) 2005 Dell Computer Corporation
  # Dual Licenced under GNU GPL and OSL
  #
  #############################################################################
"""
repository module
"""

from __future__ import generators

import os
import ConfigParser

import package
import pycompat
import dep_parser
import sys
import traceback
import firmwaretools as ft
from firmwaretools.trace_decorator import decorate, traceLog, getLog

import logging
moduleLog = getLog()
moduleVerboseLog = getLog(prefix="verbose.")

class CircularDependencyError(Exception): pass

# TODO:
#  -- conf item should NEVER be used outside of constructor (makePackage)

decorate(traceLog())
def makePackage(configFile):
    conf = ConfigParser.ConfigParser()
    conf.read(configFile)

    # make a standard package
    displayname = "unknown"
    if conf.has_option("package", "displayname"):
        displayname = conf.get("package", "displayname")

    type = package.RepositoryPackage

    try:
        pymod = conf.get("package","module")
        moduleLog.debug("pymod: %s" % pymod)
        module = __import__(pymod, globals(),  locals(), [])
        for i in pymod.split(".")[1:]:
            module = getattr(module, i)

        packageTypeClass = conf.get("package", "type")
        type = getattr(module, packageTypeClass)
        moduleLog.debug("direct instantiate")
    except (ConfigParser.NoOptionError, ConfigParser.NoSectionError, ImportError, AttributeError):
        moduleLog.debug(''.join(traceback.format_exception(sys.exc_type, sys.exc_value, sys.exc_traceback)))
        pass

    p = type(
        displayname=displayname,
        name=conf.get("package", "name"),
        version=conf.get("package", "version"),
        conf=conf,
        path=os.path.dirname(configFile),
    )

    return p

class SystemInventory(object):
    decorate(traceLog())
    def __init__(self, *args, **kargs):
        self.deviceList = {}
        self.allowDowngrade=False
        self.allowReflash=False

    decorate(traceLog())
    def addDevice(self, device):
        self.deviceList[device.uniqueInstance] = { "device": device, "update": None, "available_updates": []}

    decorate(traceLog())
    def getDevice(self, uniqueInstance, default=None):
        return self.deviceList.get(uniqueInstance, default)

    decorate(traceLog())
    def iterDevices(self, name=None):
        for device, details in self.deviceList.items():
            if name is None:
                yield details["device"]
            else:
                if details["device"].name == name:
                    yield details["device"]
                else:
                    try:
                        if details["device"].shortname == name:
                            yield details["device"]
                    except AttributeError:
                        pass

    decorate(traceLog())
    def addAvailablePackage(self, package):
        for myDev in self.iterDevices(name=package.name):
            available_updates = self.deviceList[myDev.uniqueInstance]["available_updates"]
            available_updates.append(package)
            self.deviceList[myDev.uniqueInstance]["available_updates"] = available_updates
            package.attachToDevice(myDev)

    decorate(traceLog())
    def iterAvailableUpdates(self, device):
        for pkg in self.deviceList[device.uniqueInstance]["available_updates"]:
            yield pkg

    decorate(traceLog())
    def getSuggestedUpdatePackageForDevice(self, device):
        ret = None
        if self.deviceList.has_key(device.uniqueInstance):
            ret = self.deviceList[device.uniqueInstance]["update"]
        return ret

    decorate(traceLog())
    def getUpdatePackageForDevice(self, device):
        ret = None
        if self.deviceList.has_key(device.uniqueInstance):
            if self.deviceList[device.uniqueInstance].has_key("pinned_update"):
                ret = self.deviceList[device.uniqueInstance]["pinned_update"]
            else:
                ret = self.deviceList[device.uniqueInstance]["update"]
        return ret

    decorate(traceLog())
    def pinUpdatePackage(self, device, pkg):
        #TODO: ensure that pkg is in 'available_pkgs'
        hasOldPin = False
        if self.deviceList[device.uniqueInstance].has_key("pinned_update"):
            hasOldPin = True
            oldPin = self.deviceList[device.uniqueInstance]["pinned_update"]

        self.deviceList[device.uniqueInstance]["pinned_update"] = pkg

        # just check the rules... not actually installing
        try:
            for i in self.generateInstallationOrder(): pass
        except CircularDependencyError, e:
            # roll back
            if hasOldPin:
                self.deviceList[device.uniqueInstance]["pinned_update"] = oldPin
            else:
                del(self.deviceList[device.uniqueInstance]["pinned_update"])
            raise


    decorate(traceLog())
    def unPinDevice(self, device):
        if self.deviceList[device.uniqueInstance].has_key("pinned_update"):
            del(self.deviceList[device.uniqueInstance]["pinned_update"])

    decorate(traceLog())
    def reset(self):
        for device in self.iterDevices():
            self.unPinDevice(device)

    decorate(traceLog())
    def getMemento(self, deviceHint=None):
        memento = {}
        memento['savePin'] = {}
        for deviceUniqueInstance, details in self.deviceList.items():
            if deviceHint:
                if deviceHint.uniqueInstance != deviceUniqueInstance:
                    continue
            if details.has_key("pinned_update"):
                memento['savePin'][deviceUniqueInstance] = { 'device': details["device"], 'hasPin': 1, 'oldPin': details["pinned_update"] }
            else:
                memento['savePin'][deviceUniqueInstance] = { 'device': details["device"], 'hasPin': 0, 'oldPin': None }

        memento["internal.allowReflash"] = self.allowReflash
        memento["internal.allowDowngrade"] = self.allowDowngrade
        return memento

    decorate(traceLog())
    def setMemento(self, memento):
        self.allowReflash = memento["internal.allowReflash"]
        self.allowDowngrade = memento["internal.allowDowngrade"]
        for deviceUniqueInstance, details in memento['savePin'].items():
            if details['hasPin']:
                self.pinUpdatePackage(details["device"], details["oldPin"])
            else:
                self.unPinDevice(details["device"])

    decorate(traceLog())
    def setAllowDowngrade(self, val):
        self.allowDowngrade = val

    decorate(traceLog())
    def getAllowDowngrade(self):
        return self.allowDowngrade

    decorate(traceLog())
    def setAllowReflash(self, val):
        self.allowReflash = val

    decorate(traceLog())
    def getAllowReflash(self):
        return self.allowReflash

    decorate(traceLog())
    def checkRules(self, device, candidate, unionInventory, cb=None):
        # is candidate newer than what is installed
        if not self.allowDowngrade and device.compareVersion(candidate) > 0:
            ft.callCB(cb, who="checkRules", what="package_not_newer", package=candidate, device=device)
            return 0

        # is candidate newer than what is installed
        if not self.allowReflash and device.compareVersion(candidate) == 0:
            ft.callCB(cb, who="checkRules", what="package_same_version", package=candidate, device=device)
            return 0

        #check to see if this package has specific system requirements
        # for now, check if we are on a specific system by checking for
        # a BIOS package w/ matching id. In future, may have specific
        # system package.
        if hasattr(candidate,"conf") and candidate.conf.has_option("package", "limit_system_support"):
            systemVenDev = candidate.conf.get("package", "limit_system_support")
            if not unionInventory.get( "system_bios(%s)" % systemVenDev ):
                ft.callCB(cb, who="checkRules", what="fail_limit_system_check", package=candidate)
                return 0

        #check generic dependencies
        if hasattr(candidate,"conf") and candidate.conf.has_option("package", "requires"):
            requires = candidate.conf.get("package", "requires")
            if len(requires):
                d = dep_parser.DepParser(requires, unionInventory, self.deviceList)
                if not d.depPass:
                    ft.callCB(cb, who="checkRules", what="fail_dependency_check", package=candidate, reason=d.reason)
                    return 0
        return 1


    decorate(traceLog())
    def calculateUpgradeList(self, cb=None):
        unionInventory = {}
        for deviceUniqueInstance, details in self.deviceList.items():
            unionInventory[deviceUniqueInstance] = details["device"]

        # for every device, look at the available updates to see if one can be applied.
        # if we do any work, start over so that dependencies work themselves out over multiple iterations.
        workToDo = 1
        while workToDo:
            workToDo = 0
            for deviceUniqueInstance, details in self.deviceList.items():
                for candidate in details["available_updates"]:
                    # check if this package is better than the current best
                    if unionInventory[deviceUniqueInstance].compareVersion(candidate) >= 0:
                        continue

                    if self.checkRules(details["device"], candidate, unionInventory, cb=cb):
                        self.deviceList[deviceUniqueInstance]["update"] = candidate
                        # update union inventory
                        unionInventory[deviceUniqueInstance] = candidate
                        # need another run-through in case this fixes deps for another package
                        workToDo = 1

    decorate(traceLog())
    def generateInstallationOrder(self, returnDeviceToo=0, cb=None):
        unionInventory = {}
        for deviceUniqueInstance, details in self.deviceList.items():
            unionInventory[deviceUniqueInstance] = details["device"]

        # generate initial union inventory
        # we will start with no update packages and add them in one at a time
        # as we install them
        updateDeviceList = [] # [ pkg, pkg, pkg ]
        for pkgName, details in self.deviceList.items():
            update = self.getUpdatePackageForDevice(details["device"])
            if update:
                updateDeviceList.append( (details["device"], update) )

        workToDo = 1
        while workToDo:
            workToDo = 0
            for device, candidate in updateDeviceList:
                if self.checkRules(device, candidate, unionInventory, cb=cb):
                    candidate.setCurrentInstallDevice(device)
                    if returnDeviceToo:
                        yield (device, candidate)
                    else:
                        yield candidate

                    # move pkg from to-install list to inventory list
                    updateDeviceList.remove((device,candidate))
                    unionInventory[device.uniqueInstance] = candidate

                    # need another run-through in case this fixes deps for another package
                    workToDo = 1

        if len(updateDeviceList):
            raise CircularDependencyError("packages have circular dependency, or are otherwise uninstallable.")



class Repository(object):
    decorate(traceLog())
    def __init__(self, *args):
        self.dirList = []
        for i in args:
            self.dirList.append(i)

    decorate(traceLog())
    def iterPackages(self, cb=None):
        for dir in self.dirList:
            try:
                for (path, dirs, files) in pycompat.walkPath(dir):
                    if "package.ini" in files:
                        ft.callCB(cb, who="iterPackages", what="found_package_ini", path=os.path.join(path, "package.ini" ))
                        try:
                            p = makePackage( os.path.join(path, "package.ini" ))
                            ft.callCB(cb, who="iterPackages", what="made_package", package=p)
                            yield p
                        except:
                            moduleLog.debug(''.join(traceback.format_exception(sys.exc_type, sys.exc_value, sys.exc_traceback)))
                            pass
            except OSError:   # directory doesnt exist, so no repo packages. :-)
                pass

    decorate(traceLog())
    def iterLatestPackages(self, cb=None):
        latest = {}
        for candidate in self.iterPackages(cb=cb):
            pkgName = candidate.name
            if candidate.conf.has_option("package", "limit_system_support"):
                pkgName = pkgName + "_" + candidate.conf.get("package", "limit_system_support")

            p = latest.get(pkgName)
            if not p:
                latest[pkgName] = candidate
            elif p.compareVersion(candidate) < 0:
                latest[pkgName] = candidate

        ft.callCB(cb, who="iterLatestPackages", what="done_generating_list")
        keys = latest.keys()
        keys.sort()
        for package in keys:
            ft.callCB(cb, who="iterLatestPackages", what="made_package", package=latest[package])
            yield latest[package]





# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# Copyright 2005 Duke University

import atexit
import gettext
import sys

from trace_decorator import decorate, traceLog, getLog
import errors

API_VERSION = '2.0'

# plugin can raise this to disable plugin during load
class DisablePlugin(ImportError): pass

NEXT_AVAIL_TYPE_NUM = 0
def registerPluginType(name):
    global NEXT_AVAIL_TYPE_NUM
    globals()[name] = NEXT_AVAIL_TYPE_NUM
    NEXT_AVAIL_TYPE_NUM = NEXT_AVAIL_TYPE_NUM + 1

# Plugin types
registerPluginType("TYPE_CORE")
registerPluginType("TYPE_INVENTORY")

registerPluginType("TYPE_CLI")

# testing types
registerPluginType("TYPE_MOCK_CORE")
registerPluginType("TYPE_MOCK_INVENTORY")

# all the 'normal' types
ALL_TYPES = (TYPE_CORE, TYPE_INVENTORY)

SLOT_TO_CONDUIT = {}
def registerSlotToConduit(slot, conduit):
    global SLOT_TO_CONDUIT
    SLOT_TO_CONDUIT[slot] = conduit

registerSlotToConduit('config', 'PluginConduit')
registerSlotToConduit('preinventory', 'PluginConduit')
registerSlotToConduit('inventory', 'PluginConduit')
registerSlotToConduit('postinventory', 'PluginConduit')
registerSlotToConduit('close', 'PluginConduit')

moduleLog = getLog()
moduleLogVerbose = getLog(prefix="verbose.")

class PluginExit(Exception):
    '''Used by plugins to signal to stop
    '''
    def __init__(self, value="", translation_domain=""):
        self.value = value
        self.translation_domain = translation_domain
    def __str__(self):
        if self.translation_domain:
            return gettext.dgettext(self.translation_domain, self.value)
        else:
            return self.value

class Plugins:
    '''
    Manager class for plugins.
    '''

    def __init__(self, base, optparser=None, types=None, disabled=None):
        '''Initialise the instance.
        '''
        self.base = base
        self.optparser = optparser
        self.cmdline = (None, None)

        self.verbose_logger = getLog(prefix="verbose.")

        self.disabledPlugins = disabled
        if types is None:
            types = ALL_TYPES
        if not isinstance(types, (list, tuple)):
            types = (types,)

        # TODO: load plugins here
        self._plugins = {}
        for i in self.base.listPluginsFromIni():
            conf = self.base.getPluginConfFromIni(i)
            moduleLogVerbose.info( "Checking Plugin (%s)" % i )
            if conf.enabled:
                self._loadModule(i, conf, types)

        # Call close handlers when yum exit's
        #atexit.register(self.run, 'close')

        # Let plugins register custom config file options
        self.run('config')

    decorate(traceLog())
    def _loadModule(self, pluginName, conf, types):
        # load plugin
        try:
            savePath = sys.path
            sys.path.insert(0,self.base.conf.pluginSearchPath)
            if conf.search is not None:
                sys.path.insert(0, conf.search)
            module = __import__(conf.module, globals(),  locals(), [])
            sys.path = savePath
        except DisablePlugin:
            moduleLogVerbose.info("\tPlugin raised DisablePlugin exception. skipping.")
            return
        except ImportError, e:
            sys.path = savePath
            raise errors.ConfigError(
                'Plugin "%s" cannot be loaded: %s' % (conf.module, e))

        for i in conf.module.split(".")[1:]:
            module = getattr(module, i)

        # Check API version required by the plugin
        if not hasattr(module, 'requires_api_version'):
            raise errors.ConfigError(
                'Plugin "%s" doesn\'t specify required API version' % conf.module
                )

        if not apiverok(API_VERSION, module.requires_api_version):
            raise errors.ConfigError(
                'Plugin "%s" requires API %s. Supported API is %s.' % (
                    conf.module,
                    module.requires_api_version,
                    API_VERSION,
                    ))

        # Check plugin type against filter
        plugintypes = getattr(module, 'plugin_type', None)
        if plugintypes is None:
             raise errors.ConfigError(
                'Plugin "%s" doesn\'t specify plugin type' % pluginName
                )
        if not isinstance(plugintypes, (list, tuple)):
            plugintypes = (plugintypes,)
        for plugintype in plugintypes:
            if plugintype not in types:
                moduleLogVerbose.info("\tPlugin %s not loaded: doesnt match load type (%s)" % (pluginName, plugintypes))
                return
        # Check if this plugin has been temporary disabled
        if self.disabledPlugins:
            if pluginName in self.disabledPlugins:
                moduleLogVerbose.info("\tPlugin %s not loaded: disabled" % pluginName)
                return

        moduleLogVerbose.info("\tLoaded %s plugin" % pluginName)
        self._plugins[pluginName] = {"conf": conf, "module": module}

    decorate(traceLog())
    def listLoaded(self):
        return self._plugins.keys()

    decorate(traceLog())
    def run(self, slotname, *args, **kargs):
        '''Run all plugin functions for the given slot.
        '''
        # Determine handler class to use
        conduitcls = SLOT_TO_CONDUIT.get(slotname, None)
        if conduitcls is None:
            raise ValueError('unknown slot name "%s"' % slotname)
        conduitcls = eval(conduitcls)       # Convert name to class object

        for pluginName, dets in self._plugins.items():
            module = dets['module']
            conf = dets['conf']
            hook = "%s_hook" % slotname
            if hasattr(module, hook):
                getattr(module, hook)(conduitcls(self, self.base, conf), *args, **kargs)


class DummyPlugins:
    '''
    This class provides basic emulation of the YumPlugins class. It exists so
    that calls to plugins.run() don't fail if plugins aren't in use.
    '''
    decorate(traceLog())
    def run(self, *args, **kwargs):
        pass

    decorate(traceLog())
    def setCmdLine(self, *args, **kwargs):
        pass

class PluginConduit(object):
    decorate(traceLog())
    def __init__(self, parent, base, conf):
        self._parent = parent
        self._base = base
        self._conf = conf

        self.logger = getLog()
        self.verbose_logger = getLog(prefix="verbose.")

    decorate(traceLog())
    def info(self, msg):
        self.verbose_logger.info(msg)

    decorate(traceLog())
    def error(self, msg):
        self.logger.error(msg)

    decorate(traceLog())
    def getVersion(self):
        import firmwaretools
        return firmwaretools.__version__

    decorate(traceLog())
    def getOptParser(self):
        '''Return the optparse.OptionParser instance for this execution of Yum

        In the "config" slot a plugin may add extra options to this
        instance to extend the command line options that Yum exposes.

        In all other slots a plugin may only read the OptionParser instance.
        Any modification of the instance at this point will have no effect.

        @return: the global optparse.OptionParser instance used by Yum. May be
            None if an OptionParser isn't in use.
        '''
        return self._parent.optparser

    decorate(traceLog())
    def getBase(self):
        return self._base

    decorate(traceLog())
    def getConf(self):
        return self._conf

decorate(traceLog())
def parsever(apiver):
    maj, min = apiver.split('.')
    return int(maj), int(min)

decorate(traceLog())
def apiverok(a, b):
    '''Return true if API version "a" supports API version "b"
    '''
    a = parsever(a)
    b = parsever(b)

    if a[0] != b[0]:
        return 0

    if a[1] >= b[1]:
        return 1

    return 0



#!/usr/bin/python
# vim:tw=0:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:
"""
"""

from __future__ import generators

import sys
import os
import unittest

class TestCase(unittest.TestCase):
    def setUp(self):
        if globals().get('firmwaretools'): del(firmwaretools)
        for k in sys.modules.keys():
            if k.startswith("firmwaretools"):
                del(sys.modules[k])

    def tearDown(self):
        if globals().get('firmwaretools'): del(firmwaretools)
        for k in sys.modules.keys():
            if k.startswith("firmwaretools"):
                del(sys.modules[k])

    def testCompareVersions(self):
        import firmwaretools.package as package
        self.assertEqual(-1, package.defaultCompareStrategy( "1.0", "2.0"))
        self.assertEqual( 0, package.defaultCompareStrategy( "1.0", "1.0"))
        self.assertEqual( 1, package.defaultCompareStrategy( "2.0", "1.0"))


if __name__ == "__main__":
    import test.TestLib
    sys.exit(not test.TestLib.runTests( [TestCase] ))



# vim:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:tw=0

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright (C) 2008 Dell Inc.
#  by Michael Brown <Michael_E_Brown@dell.com>

"""
Classes for subcommands of the yum command line interface.
"""

import sys

import firmwaretools.pycompat
from firmwaretools.trace_decorator import decorate, traceLog, getLog
import firmwaretools.plugins as plugins

import ftcommands
import cli

plugin_type = (plugins.TYPE_CLI,)
requires_api_version = "2.0"

moduleLog = getLog()

def config_hook(conduit, *args, **kargs):
    conduit.getOptParser().addEarlyParse("--inventory")
    conduit.getOptParser().add_option(
            "--inventory", help="List system inventory", 
            action="store_const", const="inventory", 
            dest="mode", default=None)
    conduit.getBase().registerCommand(InventoryCommand())

class InventoryCommand(ftcommands.YumCommand):
    decorate(traceLog())
    def getModes(self):
        return ['inventory']

    decorate(traceLog())
    def addSubOptions(self, base, mode, cmdline, processedArgs):
        base.optparser.add_option(
            "--show-unknown", help="Show unknown devices.", 
            action="store_true", dest="show_unknown", default=False)

    decorate(traceLog())
    def doCommand(self, base, mode, cmdline, processedArgs):
        sys.stderr.write("Wait while we inventory system:\n")

        headerWasPrinted=False
        for pkg in base.yieldInventory(cb=cli.mycb({})):
            if not headerWasPrinted:
                sys.stderr.write(firmwaretools.pycompat.clearLine())
                sys.stderr.write("System inventory:\n")
                sys.stderr.flush()
                headerWasPrinted = True

            if pkg.version == "unknown" and not base.opts.show_unknown :
                continue

            print("\t%s = %s" % (str(pkg), pkg.version))

        return [0, "Done"]




#!/usr/bin/python
# vim:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:
"""
"""

from __future__ import generators

import sys
import os
import unittest

class TestCase(unittest.TestCase):
    def setUp(self):
        if globals().get('firmwaretools'): del(firmwaretools)
        for k in sys.modules.keys():
            if k.startswith("firmwaretools"):
                del(sys.modules[k])

        import firmwaretools.package as package
        pkgA = package.Package(name="pkgA", version="a01", displayname="test pkgA")
        pkgB = package.Package(name="pkgC", version="a02", displayname="test pkgC")
        pkgC = package.Package(name="pkgC", version="a03", displayname="test pkgC")

        self.inventory = {}
        self.inventory['pkgA'] = pkgA
        self.inventory['pkgB'] = pkgB
        self.inventory['pkgC'] = pkgC

    def tearDown(self):
        if globals().get('firmwaretools'): del(firmwaretools)
        for k in sys.modules.keys():
            if k.startswith("firmwaretools"):
                del(sys.modules[k])

    def testExist(self):
        import firmwaretools.dep_parser as dep_parser
        s = "pkgA"
        d = dep_parser.DepParser(s, self.inventory, {})
        self.assertEquals(1, d.depPass)

    def testExist2(self):
        import firmwaretools.dep_parser as dep_parser
        tests = [ ("exist", "pkgA"),
            ("gt",  "pkgA > a00"),
            ("ge1", "pkgA >= a00"),
            ("ge2", "pkgA >= a01"),
            ("eq",  "pkgA == a01"),
            ("le1", "pkgA <= a01"),
            ("le2", "pkgA <= a02"),
            ("lt",  "pkgA < a02"),
            ("exist_gt", "pkgA, pkgB > a01"),
            ("exist_gt_exist", "pkgA, pkgB > a01, pkgC"), ]

        for name, testStr in tests:
            d = dep_parser.DepParser(testStr, self.inventory, {})
            self.assertEquals(1, d.depPass)

    def testExist3(self):
        import firmwaretools.dep_parser as dep_parser
        tests = [ ("exist", "pkgD"),
            ("gt",  "pkgA > a01"),
            ("ge1", "pkgA >= a02"),
            ("ge2", "pkgA >= a03"),
            ("eq",  "pkgA == a02"),
            ("le1", "pkgA <= a00"),
            ("lt",  "pkgA < a01"),
            ("exist_gt", "pkgA, pkgB > a01, pkgD"),
            ("exist_gt_exist", "pkgA, pkgB < a01, pkgC"), ]

        for name, testStr in tests:
            d = dep_parser.DepParser(testStr, self.inventory, {})
            self.assertEquals(0, d.depPass)






if __name__ == "__main__":
    import test.TestLib
    sys.exit(not test.TestLib.runTests( [TestCase] ))



#!/usr/bin/python
# vim:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:
"""
"""

from __future__ import generators

import sys
import os
import unittest

class TestCase(unittest.TestCase):
    def setUp(self):
        if globals().get('firmwaretools'): del(firmwaretools)
        for k in sys.modules.keys():
            if k.startswith("firmwaretools"):
                del(sys.modules[k])

    def tearDown(self):
        if globals().get('firmwaretools'): del(firmwaretools)
        for k in sys.modules.keys():
            if k.startswith("firmwaretools"):
                del(sys.modules[k])

    def testExcCommandNoTimeout(self):
        import firmwaretools.pycompat as pycompat
        pycompat.executeCommand("sleep 0", timeout=0)

    def testExcCommandTimeout(self):
        import firmwaretools.pycompat as pycompat
        self.assertRaises(pycompat.commandTimeoutExpired, pycompat.executeCommand, "sleep 3", timeout=1)

    def testExcCommandAlarmNoTimeout(self):
        # test that executeCommand() doesn't interfere with existing alarm calls
        # given a command that itself doesnt timeout
        import signal, time
        import firmwaretools.pycompat as pycompat
        class alarmExc(Exception): pass
        def alarmhandler(signum,stackframe):
            raise alarmExc("timeout expired")

        oldhandler=signal.signal(signal.SIGALRM,alarmhandler)
        prevTimeout = signal.alarm(1)
        pycompat.executeCommand("sleep 0", timeout=1)
        self.assertRaises(alarmExc, time.sleep, 5)

    def testExcCommandAlarmTimeout(self):
        # test that executeCommand() doesn't interfere with existing alarm calls
        # given a command that itself times out
        import signal, time
        import firmwaretools.pycompat as pycompat
        class alarmExc(Exception): pass
        def alarmhandler(signum,stackframe):
            raise alarmExc("timeout expired")

        oldhandler=signal.signal(signal.SIGALRM,alarmhandler)
        prevTimeout = signal.alarm(2)
        self.assertRaises(pycompat.commandTimeoutExpired, pycompat.executeCommand,"sleep 2", timeout=1)
        self.assertRaises(alarmExc, time.sleep, 10)



if __name__ == "__main__":
    import test.TestLib
    sys.exit(not test.TestLib.runTests( [TestCase] ))



# vim:tw=0:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:

  #############################################################################
  #
  # Copyright (c) 2005 Dell Computer Corporation
  # Dual Licenced under GNU GPL and OSL
  #
  #############################################################################
"""
package module
"""

import rpm
from gettext import gettext as _

from firmwaretools.trace_decorator import decorate, traceLog, getLog

class InternalError(Exception): pass
class InstallError(Exception): pass
class NoInstaller(Exception): pass

def defaultCompareStrategy(ver1, ver2):
    return rpm.labelCompare( ("0", str(ver1), "0"), ("0", str(ver2), "0"))

packageStatusEnum = {
    "unknown": _("The package status is not known."),
    "not_installed": _("The device has not been updated to this version."),
    "in_progress":   _("The device is being updated now"),
    "failed":        _("Device update failed."),
    "success":       _("Device update was successful."),
    "disabled":       _("Device update is disabled for this device."),
    "warm_reboot_needed":       _("Update complete. You must perform a warm reboot for the update to take effect."),
    }

# Package public API:
#   pkg.name
#   pkg.version
#   str(pkg) == display name
#   pkg.compareVersion(otherPkg)
class Package(object):
    def __init__(self, *args, **kargs):
        self.name = None
        self.version = None
        self.compareStrategy = defaultCompareStrategy
        for key, value in kargs.items():
            setattr(self, key, value)

        assert(hasattr(self, "name"))
        assert(hasattr(self, "version"))
        assert(hasattr(self, "displayname"))
        assert(len(self.name))
        assert(len(self.version))
        assert(len(self.displayname))

        status = "unknown"

    def __str__(self):
        if hasattr(self, "displayname"):
            return self.displayname
        return self.name

    def compareVersion(self, otherPackage):
        return self.compareStrategy(self.version, otherPackage.version)

class RepositoryPackage(Package):
    mainIni = None
    def __init__(self, *args, **kargs):
        self.installFunction = None
        self.path = None
        super(RepositoryPackage, self).__init__(*args, **kargs)

        self.capabilities = {
            # if package is capable of downgrading
            'can_downgrade': False,

            # if package is capable of reflashing same version
            'can_reflash': False,

            # if package has/updates .percent_done member var
            # GUI can use progress bar if this is set.
            # otherwise, GUI should just use a spinner or something
            'accurate_update_percentage': False,

            # if update has .update_status_text member var
            # GUI should use for 'view log' function
            'update_log_string': False,

            # if update has .update_status_logfile member var
            # GUI should use for 'view log' function
            'update_log_filename': False,
            }

        self.progressPct = 0
        self.status = "not_installed"
        self.deviceList = []
        self.currentInstallDevice = None

    def getProgress(self):
        # returns real number between 0-1, or -1 for "not supported"
        if self.capabilities['accurate_update_percentage']:
            return self.progressPct
        else:
            return -1

    def install(self):
        self.status = "in_progress"
        if self.installFunction is not None:
            return self.installFunction(self)

        self.status = "failed"
        raise NoInstaller(_("Attempt to install a package with no install function. Name: %s, Version: %s") % (self.name, self.version))

    def getCapability(self, capability):
        return self.capabilities.get(capability, None)

    def attachToDevice(self, device):
        self.deviceList.append(device)

    def getDeviceList(self):
        return self.deviceList

    def setCurrentInstallDevice(self, device):
        self.currentInstallDevice = device

    def getCurrentInstallDevice(self):
        return self.currentInstallDevice

    def getStatusStr(self):
        return packageStatusEnum.get(self.status, _("Programming error: status code not found."))


# Base class for all devices on a system
# required:
#   displayname
#   name
#   version
# optional:
#   compareStrategy
class Device(Package):
    def __init__(self, *args, **kargs):
        self.name = None
        self.version = None
        self.compareStrategy = defaultCompareStrategy
        for key, value in kargs.items():
            setattr(self, key, value)

        if not hasattr(self, "uniqueInstance"):
            self.uniqueInstance = self.name

        assert(hasattr(self, "name"))
        assert(hasattr(self, "version"))
        assert(hasattr(self, "displayname"))

        status = "unknown"

    def __str__(self):
        if hasattr(self, "displayname"):
            return self.displayname
        return self.name

    def compareVersion(self, otherPackage):
        return self.compareStrategy(self.version, otherPackage.version)


# required:  (in addition to base class)
#   pciDbdf
class PciDevice(Device):
    def __init__(self, *args, **kargs):
        super(Device, self).__init__(*args, **kargs)
        assert(hasattr(self, "pciDbdf"))
        self.uniqueInstance = "pci_dev_at_domain_0x%04x_bus_0x%02x_dev_0x%02x_func_0x%01x" % self.pciDbdf





# vim:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:tw=0

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright (C) 2008 Dell Inc.
#  by Michael Brown <Michael_E_Brown@dell.com>

"""
Classes for subcommands of the yum command line interface.
"""

from firmwaretools.trace_decorator import decorate, traceLog, getLog
import firmwaretools.plugins as plugins

import ftcommands

plugin_type = (plugins.TYPE_CLI,)
requires_api_version = "2.0"

moduleLog = getLog()

def config_hook(conduit, *args, **kargs):
    conduit.getOptParser().addEarlyParse("--listplugins")
    conduit.getOptParser().add_option(
        "--listplugins", action="store_const", const="listplugins", 
        dest="mode", help="list available plugins.")
    conduit.getBase().registerCommand(ListPluginsCommand())

class ListPluginsCommand(ftcommands.YumCommand):
    decorate(traceLog())
    def getModes(self):
        return ['listplugins']

    decorate(traceLog())
    def doCommand(self, base, mode, cmdline, processedArgs):
        print("Available Plugins:")
        for p in base.listPluginsFromIni():
            print("\t%s" % p)

        print("Loaded Plugins:")
        for p in base.plugins.listLoaded():
            print("\t%s" % p)

        return [0, "Done"]





# vim:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:tw=0

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright (C) 2008 Dell Inc.
#  by Michael Brown <Michael_E_Brown@dell.com>
#
# Based in part on design and code in Yum 3.2:
# Copyright 2006 Duke University
# Written by Seth Vidal


"""
Command line interface class and related.
"""

import os
import re
import sys
import time
import random
import logging

from optparse import OptionParser

import firmwaretools
import firmwaretools.plugins as plugins
from firmwaretools.trace_decorator import decorate, traceLog, getLog

from firmwaretools.i18n import _
import signal
import ftcommands
from firmwaretools import errors

__VERSION__=firmwaretools.__VERSION__

def sigquit(signum, frame):
    """ SIGQUIT handler for the cli. """
    print >> sys.stderr, "Quit signal sent - exiting immediately"
    sys.exit(1)

class CliError(errors.BaseError): pass

class BaseCli(firmwaretools.FtBase):
    """This is the base class for cli.
       Inherits from FtBase """

    def __init__(self):
        # handle sigquit early on
        signal.signal(signal.SIGQUIT, sigquit)
        logging.basicConfig()
        logging.raiseExceptions = 0
        firmwaretools.FtBase.__init__(self)

        self.logger = getLog()
        self.verbose_logger = getLog(prefix="verbose.")

        self.cli_commands = {}

    def registerCommand(self, command):
        for name in command.getModes():
            if self.cli_commands.has_key(name):
                raise errors.ConfigError('Command "%s" already defined' % name)
            self.cli_commands[name] = command

    def getOptionsConfig(self, args):
        """parses command line arguments, takes cli args:
        sets up self.conf and self.cmds as well as logger objects
        in base instance"""

        self.fullCmdLine = args

        self.optparser = FtOptionParser( usage='%prog [options]', version=__VERSION__)

        # Parse only command line options that affect basic yum setup
        self.args = []
        self.opts = self.optparser.firstParse(args)

        self.verbosity = self.opts.verbosity
        self.trace = self.opts.trace
        self.loggingConfig = self.opts.configFiles[0]

        pluginTypes = [plugins.TYPE_CLI, plugins.TYPE_CORE]
        if not self.opts.fake_mode:
            pluginTypes.extend([plugins.TYPE_INVENTORY,])
        else:
            pluginTypes.extend([plugins.TYPE_MOCK_CORE, plugins.TYPE_MOCK_INVENTORY,])

        # Read up configuration options and initialise plugins
        try:
            self._getConfig(self.opts.configFiles,
                    pluginTypes,
                    optparser=self.optparser,
                    disabledPlugins=self.opts.disabledPlugins)

        except errors.ConfigError, e:
            self.logger.critical(_('Config Error: %s'), e)
            sys.exit(1)
        except ValueError, e:
            self.logger.critical(_('Options Error: %s'), e)
            sys.exit(1)

        # redo firstparse in case plugin added a new mode
        self.opts = self.optparser.firstParse(args)

        # subcommands can add new optparser stuff in addSubOptions()
        self.doCommands("addSubOptions")

        # Now parse the command line for real and
        self.opts, self.args = self.optparser.parse_args(args)

        # check fully-processed cmdline options
        self.doCommands("doCheck")

    decorate(traceLog())
    def doCommands(self, funcName="doCommand"):
        if not self.cli_commands.has_key(self.opts.mode):
            self.usage()
            raise CliError, "mode not specified."

        return getattr(self.cli_commands[self.opts.mode], funcName)(self,
            self.opts.mode, self.fullCmdLine, self.args)


    decorate(traceLog())
    def usage(self):
        ''' Print out command line usage '''
        self.optparser.print_help()

    decorate(traceLog())
    def updateFirmware(self, showUnknown=False):
        print

        print "Running system inventory..."
        depFailures = {}
        updateSet = self.calculateUpgradeList(cb=mycb(depFailures))

        print "\033[2K\033[0G"  # clear line
        needUpdate = 0
        print "Searching storage directory for available BIOS updates..."
        for device in updateSet.iterDevices():
            if device.version == "unknown" and not showUnknown:
                continue

            print "Checking %s - %s" % (str(device), device.version)
            for availPkg in updateSet.iterAvailableUpdates(device):
                print "\tAvailable: %s - %s" % (availPkg.name, availPkg.version)

            pkg = updateSet.getUpdatePackageForDevice(device)
            if pkg is None:
                print "\tDid not find a newer package to install that meets all installation checks."
            else:
                print "\tFound Update: %s - %s" % (pkg.name, pkg.version)
                needUpdate = 1

        if depFailures:
            print
            print "Following packages could apply, but have dependency failures:"

        for pkg, reason in depFailures.values():
            print "\t%s - %s" % (pkg.name, pkg.version)
            print "\t\t REASON: %s" % reason

        if not needUpdate:
            print
            print "This system does not appear to have any updates available."
            print "No action necessary."
            print
            return 1
        else:
            print
            print "Found firmware which needs to be updated."
            print

        # if we get to this point, that means update is necessary.
        # any exit before this point means that there was an error, or no update
        # was necessary and should return non-zero
        if self.opts.interactive == 2:
            print
            print "Test mode complete."
            print
            return 0

        if self.opts.interactive == 1:
            print
            print "Please run the program with the '--yes' switch to enable BIOS update."
            print "   UPDATE NOT COMPLETED!"
            print
            return 0

        print "Running updates..."
        for pkg in updateSet.generateInstallationOrder():
            try:

                def statusFunc():
                    if pkg.getCapability('accurate_update_percentage'):
                        firmwaretools.pycompat.spinPrint("%s%% Installing %s - %s" % (pkg.getProgress() * 100, pkg.name, pkg.version))
                    else:
                        firmwaretools.pycompat.spinPrint("Installing %s - %s" % (pkg.name, pkg.version))
                    time.sleep(0.2)

                ret = firmwaretools.pycompat.runLongProcess(pkg.install, waitLoopFunction=statusFunc)
                print firmwaretools.pycompat.clearLine(),
                print "100%% Installing %s - %s" % (pkg.name, pkg.version)
                print "Done: %s" % pkg.getStatusStr()
                print

            except (firmwaretools.package.NoInstaller,), e:
                print "package %s - %s does not have an installer available." % (pkg.name, pkg.version)
                print "skipping this package for now."
                continue
            except (firmwaretools.package.InstallError,), e:
                print "Installation failed for package: %s - %s" % (pkg.name, pkg.version)
                print "aborting update..."
                print
                print "The error message from the low-level command was:"
                print
                print e
                break


class mycb(firmwaretools.Callback):
    def __init__(self, depFailures):
        super(mycb, self).__init__()
        self.depFailures = depFailures
        self.message = ""

    def __call__(self, *args, **kargs):
        return super(mycb, self).__call__(*args, **kargs)

    def UNKNOWN(self, *args, **kargs):
        firmwaretools.pycompat.spinPrint(self.message)

    def running_inventory(self, who, what, details, *args, **kargs):
        self.message = "%s: Running Inventory: %s" % (who, details)
        firmwaretools.pycompat.spinPrint("%s: Running Inventory: %s" % (who, details))

    def found_package_ini(what, path, *args, **kargs):
        if len(path) > 50:
            path = path[-50:]
        self.message = "Checking: %s" % path
        firmwaretools.pycompat.spinPrint("Checking: %s" % path)

    def fail_dependency_check(self, what, package, *args, **kargs):
        pkgName = "%s-%s" % (package.name, package.version)
        if package.conf.has_option("package", "limit_system_support"):
            pkgName = pkgName + "-" + package.conf.get("package", "limit_system_support")
        self.depFailures[pkgName] = (kargs.get("package"), kargs.get("reason"))


class FtOptionParser(OptionParser):
    """Unified cmdline option parsing and config file handling."""
    def __init__(self, *args, **kargs):
        OptionParser.__init__(self, *args, **kargs)

        self.add_option("-c", "--config", help="Override default config file with user-specified config file.", dest="configFiles", action="append", default=[])
        self.add_option("--extra-plugin-config", help="Add additional plugin config file.", action="append", default=[], dest="extraConfigs")
        self.add_option("-v", "--verbose", action="count", dest="verbosity", default=1, help="Display more verbose output.")
        self.add_option("-q", "--quiet", action="store_const", const=0, dest="verbosity", help="Minimize program output. Only errors and warnings are displayed.")
        self.add_option("--trace", action="store_true", dest="trace", default=False, help="Enable verbose function tracing.")
        self.add_option("--fake-mode", action="store_true", dest="fake_mode", default=False, help="Display fake data for unit-testing.")
        self.add_option("--disableplugin", action="append", dest="disabledPlugins", default=[], help="Disable single named plugin.")

        # put all 'mode' arguments here so we know early what mode we are in.
        self.parseOptionsFirst_novalopts = [
            "--version", "-q", "-v", "--quiet", "--verbose",
            "--trace", "--fake-mode", ]
        self.parseOptionsFirst_valopts = [
            "-c", "--config", "--disableplugin", "--extra-plugin-config"]

    def addEarlyParse(self, opt, arg=0):
        if arg:
            self.parseOptionsFirst_valopts.append(opt)
        else:
            self.parseOptionsFirst_novalopts.append(opt)

    def firstParse(self, args):
        args = _filtercmdline(
            self.parseOptionsFirst_novalopts,
            self.parseOptionsFirst_valopts,
            args)
        opts, args = self.parse_args(args=args)

        if not opts.configFiles:
            opts.configFiles = [os.path.join(firmwaretools.PKGCONFDIR, "firmware.conf"), ]

        opts.configFiles = opts.configFiles + opts.extraConfigs

        return opts

def _filtercmdline(novalopts, valopts, args):
    '''Keep only specific options from the command line argument list

    This function allows us to peek at specific command line options when using
    the optparse module. This is useful when some options affect what other
    options should be available.

    @param novalopts: A sequence of options to keep that don't take an argument.
    @param valopts: A sequence of options to keep that take a single argument.
    @param args: The command line arguments to parse (as per sys.argv[:1]
    @return: A list of strings containing the filtered version of args.

    Will raise ValueError if there was a problem parsing the command line.
    '''
    out = []
    args = list(args)       # Make a copy because this func is destructive

    while len(args) > 0:
        a = args.pop(0)
        if '=' in a:
            opt, _ = a.split('=', 1)
            if opt in valopts:
                out.append(a)

        elif a in novalopts:
            out.append(a)

        elif a in valopts:
            if len(args) < 1:
                raise ValueError
            next = args.pop(0)
            if next[0] == '-':
                raise ValueError

            out.extend([a, next])

        else:
            # Check for single letter options that take a value, where the
            # value is right up against the option
            for opt in valopts:
                if len(opt) == 2 and a.startswith(opt):
                    out.append(a)

    return out



# vim:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:tw=0

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# Copyright 2005 Duke University 

"""
Entrance point for the command line interface.
"""

import sys
import locale
import logging
import signal
import time # test purposes only

# fixup 'no handlers could be found for...' message
logging.raiseExceptions = 0

from firmwaretools.trace_decorator import decorate, traceLog, getLog
from firmwaretools import errors
from firmwaretools import plugins
import cli

def main(args):
    """This does all the real work"""
    def setDebug():
        import pdb
        pdb.set_trace()

    signal.signal(signal.SIGUSR1,setDebug)

    def exUserCancel():
        logger.critical('Exiting on user cancel')
        sys.exit(1)

    decorate(traceLog())
    def exIOError(e):
        if e.errno == 32:
            logger.critical('Exiting on Broken Pipe')
        else:
            logger.critical(str(e))
        sys.exit(1)

    decorate(traceLog())
    def exPluginExit(e):
        '''Called when a plugin raises PluginExit.

        Log the plugin's exit message if one was supplied.
        '''
        if str(e):
            logger.warn('%s' % e)
        sys.exit(1)

    decorate(traceLog())
    def exFatal(e):
        logger.critical('%s' % e)
        sys.exit(1)


    try:
        locale.setlocale(locale.LC_ALL, '')
    except locale.Error, e:
        # default to C locale if we get a failure.
        print >> sys.stderr, 'Failed to set locale, defaulting to C'
        locale.setlocale(locale.LC_ALL, 'C')
        
    # our core object for the cli
    base = cli.BaseCli()

    logger = getLog()
    verbose_logger = getLog(prefix="verbose.")

    # do our cli parsing and config file setup
    # also sanity check the things being passed on the cli
    try:
        # no logging before this returns.
        base.getOptionsConfig(args)
    except plugins.PluginExit, e:
        exPluginExit(e)
    except errors.BaseError, e:
        exFatal(e)

    lockerr = ""
    while True:
        try:
            base.lock()
        except errors.LockError, e:
            if "%s" %(e.msg,) != lockerr:
                lockerr = "%s" %(e.msg,)
                logger.critical(lockerr)
            logger.critical("Another app is currently holding the lock; waiting for it to exit...")
            time.sleep(2)
        else:
            break

    try:
        result, resultmsgs = base.doCommands()
    except plugins.PluginExit, e:
        exPluginExit(e)
    except errors.BaseError, e:
        result = 1
        resultmsgs = [str(e)]
    except KeyboardInterrupt:
        exUserCancel()
    except IOError, e:
        exIOError(e)

    verbose_logger.info('Complete!')
    base.unlock()
    sys.exit(0)


if __name__ == "__main__":
    try:
        main(sys.argv[1:])
    except KeyboardInterrupt, e:
        print >> sys.stderr, "\n\nExiting on user cancel."
        sys.exit(1)



# vim:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:tw=0

import exceptions

class BaseError(exceptions.Exception): pass

class ConfigError(BaseError): pass
class LockError(BaseError): pass



# vim:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:tw=0

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright (C) 2008 Dell Inc.
#  by Michael Brown <Michael_E_Brown@dell.com>

"""
Classes for subcommands of the yum command line interface.
"""

from firmwaretools.trace_decorator import decorate, traceLog, getLog
import firmwaretools.plugins as plugins

import ftcommands

plugin_type = (plugins.TYPE_CLI,)
requires_api_version = "2.0"

moduleLog = getLog()

def config_hook(conduit, *args, **kargs):
    conduit.getOptParser().addEarlyParse("--bootstrap")
    conduit.getOptParser().add_option(
        "-b", "--bootstrap", help="List the bootstrap inventory",
        action="store_const", const="bootstrap", dest="mode", default=None)
    conduit.getBase().registerCommand(BootstrapCommand())

class BootstrapCommand(ftcommands.YumCommand):
    decorate(traceLog())
    def getModes(self):
        return ['bootstrap']

    decorate(traceLog())
    def addSubOptions(self, base, mode, cmdline, processedArgs):
        # need to add bootstrap-specific options to optparser
        base.optparser.add_option("-u", "--up2date_mode", action="store_true", dest="comma_separated", default=False, help="Comma-separate values for use with up2date.")
        base.optparser.add_option("-a", "--apt_mode", action="store_true", dest="apt_mode", default=False, help="fixup names so that they are compatible with apt")

    decorate(traceLog())
    def doCommand(self, base, mode, cmdline, processedArgs):
        parse=str
        if base.opts.apt_mode:
            parse = debianCleanName

        venId, sysId = base.getSystemId()

        out = ""
        for pkg in base.yieldInventory():
            if base.opts.comma_separated:
                if venId and sysId:
                    out = out + ",%s" % parse(pkg.name + "/system(ven_0x%04x_dev_0x%04x)" % (venId, sysId))
                out = out + ",%s" % parse(pkg.name)
                try:
                    if venId and sysId:
                        out = out + ",%s" % parse(pkg.shortname + "/system(ven_0x%04x_dev_0x%04x)" % (venId, sysId))
                    out = out + ",%s" % parse(pkg.shortname)
                except AttributeError:
                    pass
            else:
                if venId and sysId:
                    print("%s/system(ven_0x%04x_dev_0x%04x)" % (parse(pkg.name), venId, sysId))
                print("%s" % parse(pkg.name))
                try:
                    if venId and sysId:
                        print("%s/system(ven_0x%04x_dev_0x%04x)" % (parse(pkg.shortname), venId, sysId))
                    print("%s" % parse(pkg.shortname))
                except AttributeError:
                    pass


        # strip leading comma:
        out = out[1:]
        if out:
            print(out)

        return [0, "Done"]


# used by bootstrap
decorate(traceLog())
def debianCleanName(s):
    s = s.replace('_', '-')
    s = s.replace('(', '-')
    s = s.replace(')', '')
    s = s.lower()
    return s




#!/usr/bin/python
# vim:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:
"""
"""

from __future__ import generators

import sys
import unittest
import ConfigParser
import os

class TestCase(unittest.TestCase):
    def setUp(self):
        if globals().get('firmwaretools'): del(firmwaretools)
        for k in sys.modules.keys():
            if k.startswith("firmwaretools"):
                del(sys.modules[k])

    def tearDown(self):
        if globals().get('firmwaretools'): del(firmwaretools)
        for k in sys.modules.keys():
            if k.startswith("firmwaretools"):
                del(sys.modules[k])

    def testBootstrapInventory(self):
        import firmwaretools
        import firmwaretools.plugins as plugins

        # manually setup fake config file
        f = firmwaretools.FtBase()
        pluginTypes = [
            plugins.TYPE_CORE,
            plugins.TYPE_MOCK_CORE, plugins.TYPE_MOCK_INVENTORY,
            ] 
        #instead of "f._getConfig(None, pluginTypes, None, [])"
        # do the below to avoid "logging.config.fileConfig(configFile)"
        cfgFiles = [os.path.join(firmwaretools.PKGCONFDIR, "firmware.conf"),]
        f.conf = firmwaretools.confObj()
        f.setConfFromIni(cfgFiles)
        f.conf.uid = os.geteuid()
        f.doPluginSetup(None , pluginTypes, [])

        # import functions for bootstrap/compare

        # run bootstrap and compare.
        index = 0
        for pkg in f.yieldInventory():
            self.assertEqual( firmwaretools.mockpackage.mockExpectedOutput.split("\n")[index], pkg.name )
            index = index + 1

        # ensure it actually ran.
        self.assertEqual(index, len(firmwaretools.mockpackage.mockExpectedOutput.split("\n")))


if __name__ == "__main__":
    import test.TestLib
    sys.exit(not test.TestLib.runTests( [TestCase] ))



# vim:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:tw=0

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright (C) 2008 Dell Inc.
#  by Michael Brown <Michael_E_Brown@dell.com>
#
# Based in part on design and code in Yum 3.2:
# Copyright 2006 Duke University 
# Written by Seth Vidal

"""
Classes for subcommands of the yum command line interface.
"""

from firmwaretools.trace_decorator import decorate, traceLog, getLog

moduleLog = getLog()

class YumCommand(object):
        
    def getModes(self):
        return []

    def doCheck(self, base, mode, cmdline, processedArgs):
        pass

    def addSubOptions(self, base, mode, cmdline, processedArgs):
        pass

    def doCommand(self, base, mode, cmdline, processedArgs):
        """
        @return: (exit_code, [ errors ]) where exit_code is:
           0 = we're done, exit
           1 = we've errored, exit with error string
        """
        return 0, ['Nothing to do']
    




#!/usr/bin/python
# vim:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:tw=0
"""
"""

from __future__ import generators

import sys
import os
import unittest

try:
    test_path = os.path.dirname( os.path.realpath( globals()["__file__"] ) )
except KeyError:
    test_path = os.path.realpath( sys.path[0] )

datafiles = os.path.join( test_path, "datafiles" )

def generateUpdateSet(repo, inv):
    import firmwaretools.repository as repository
    _systemInventory = repository.SystemInventory()
    for dev in inv:
        _systemInventory.addDevice(dev)

    for candidate in repo.iterPackages():
        _systemInventory.addAvailablePackage(candidate)

    _systemInventory.calculateUpgradeList()
    return _systemInventory


class TestCase(unittest.TestCase):
    def setUp(self):
        if globals().get('firmwaretools'): del(firmwaretools)
        for k in sys.modules.keys():
            if k.startswith("firmwaretools"):
                del(sys.modules[k])

    def tearDown(self):
        if globals().get('firmwaretools'): del(firmwaretools)
        for k in sys.modules.keys():
            if k.startswith("firmwaretools"):
                del(sys.modules[k])

    def testRepositoryInventory(self):
        import firmwaretools.repository as repository
        r = repository.Repository(datafiles)
        for pkg in r.iterPackages():
            pass

    def testIterLatest(self):
        import firmwaretools.repository as repository
        r = repository.Repository(datafiles)
        list = []

        # list of versions for system_specific
        li = ["a04", "a09"]

        for pkg in r.iterLatestPackages():
            if pkg.name == "testpack":
                self.assertEqual( pkg.version, "a06" )
            elif pkg.name == "testpack_another":
                self.assertEqual( pkg.version, "a04" )
            elif pkg.name == "testpack_different":
                self.assertEqual( pkg.version, "a07" )
            elif pkg.name == "testpack_newpkgstrat":
                self.assertEqual( pkg.version, "a08" )
            elif pkg.name == "system_specific":
                self.failUnless( pkg.version in li )
                li.remove(pkg.version)
            elif pkg.name == "test_requires":
                self.assertEqual( pkg.version, "a09" )
            elif pkg.name == "testorder1":
                self.assertEqual( pkg.version, "a04" )
            elif pkg.name == "testorder2":
                self.assertEqual( pkg.version, "a05" )
            elif pkg.name == "testorder3":
                self.assertEqual( pkg.version, "a06" )
            elif pkg.name == "circtest1":
                self.assertEqual( pkg.version, "a04" )
            elif pkg.name == "circtest2":
                self.assertEqual( pkg.version, "a04" )
            else:
                self.fail("Unknown package. %s version %s" % (pkg.name, pkg.version))

        self.assertEqual( len(li), 0 )

    def testGenerateUpdateSet1(self):
        import firmwaretools.repository as repository
        import firmwaretools.package as package
        p = package.Device(
            name = "testpack",
            version = "a04",
            displayname = "fake"
            )
        systemInventory = [p,]
        r = repository.Repository(datafiles)
        updateSet = generateUpdateSet(r, systemInventory)

        self.assertEqual( updateSet.getUpdatePackageForDevice(p).name, "testpack" )
        self.assertEqual( updateSet.getUpdatePackageForDevice(p).version, "a06" )

    def testGenerateUpdateSet2(self):
        import firmwaretools.repository as repository
        import firmwaretools.package as package
        p = package.Device(
            name = "testpack_different",
            version = "a04",
            displayname = "fake"
            )
        systemInventory = [p,]
        r = repository.Repository(datafiles)
        updateSet = generateUpdateSet(r, systemInventory)

        self.assertEqual( updateSet.getUpdatePackageForDevice(p).name, "testpack_different" )
        self.assertEqual( updateSet.getUpdatePackageForDevice(p).version, "a07" )

    def testGenerateUpdateSet3(self):
        import firmwaretools.repository as repository
        import firmwaretools.package as package
        p = package.Device(
            name = "testpack",
            version = "a08",
            displayname = "fake"
            )
        systemInventory = [p,]
        r = repository.Repository(datafiles)
        updateSet = generateUpdateSet(r, systemInventory)

        self.failUnless( updateSet.getUpdatePackageForDevice(p) is None )

    def testGenerateUpdateSet4_andInstall(self):
        import firmwaretools.repository as repository
        import firmwaretools.package as package
        p = package.Device(
            name = "testpack_newpkgstrat",
            version = "a04",
            displayname = "fake"
            )
        systemInventory = [p,]
        r = repository.Repository(datafiles)
        updateSet = generateUpdateSet(r, systemInventory)

        self.assertEqual( updateSet.getUpdatePackageForDevice(p).name, "testpack_newpkgstrat" )
        self.assertEqual( updateSet.getUpdatePackageForDevice(p).version, "a08")
        res = updateSet.getUpdatePackageForDevice(p).install()
        self.assertEqual( res, "SUCCESS" )

    def testGenerateUpdateSetMultiple(self):
        import firmwaretools.repository as repository
        import firmwaretools.package as package

        p = package.Device(
            name = "testpack",
            version = "a04",
            displayname = "fake"
            )

        q = package.Device(
            name = "testpack_different",
            version = "a04",
            displayname = "fake"
            )

        r = package.Device(
            name = "testpack_another",
            version = "a05",
            displayname = "fake"
            )

        systemInventory = [p,q,r]
        repo = repository.Repository(datafiles)
        updateSet = generateUpdateSet(repo, systemInventory)

        self.assertEqual( updateSet.getUpdatePackageForDevice(p).name, "testpack" )
        self.assertEqual( updateSet.getUpdatePackageForDevice(p).version, "a06" )
        self.assertEqual( updateSet.getUpdatePackageForDevice(q).name, "testpack_different" )
        self.assertEqual( updateSet.getUpdatePackageForDevice(q).version, "a07" )
        self.failUnless( updateSet.getUpdatePackageForDevice(r) is None )

    def testGenerateUpdateSetInstallDefault(self):
        import firmwaretools.repository as repository
        import firmwaretools.package as package
        p = package.Device(
            name = "testpack",
            version = "a04",
            displayname = "fake"
            )
        systemInventory = [p,]
        r = repository.Repository(datafiles)
        updateSet = generateUpdateSet(r, systemInventory)
        self.assertRaises(package.NoInstaller, updateSet.getUpdatePackageForDevice(p).install)

    def testGenerateUpdateSetInstall(self):
        import firmwaretools.repository as repository
        import firmwaretools.package as package
        p = package.Device(
            name = "testpack_different",
            version = "a04",
            displayname = "fake"
            )
        systemInventory = [p,]
        r = repository.Repository(datafiles)
        updateSet = generateUpdateSet(r, systemInventory)
        res = updateSet.getUpdatePackageForDevice(p).install()
        self.assertEqual( res, "SUCCESS" )


    def testGenerateUpdateSet_SystemSpecific1(self):
        # test the case where system specific update available that doesn't
        # apply to current system
        import firmwaretools.repository as repository
        import firmwaretools.package as package
        p = package.Device(
            name = "system_specific",
            version = "a01",
            displayname = "fake"
            )
        systemInventory = [p,]
        r = repository.Repository(datafiles)
        updateSet = generateUpdateSet(r, systemInventory)

        self.failUnless( updateSet.getUpdatePackageForDevice(p) is None )

    def testGenerateUpdateSet_SystemSpecific2(self):
        # test the case where system specific update available that does
        # apply to current system
        import firmwaretools.repository as repository
        import firmwaretools.package as package

        p = package.Device(
            name = "system_specific",
            version = "a01",
            displayname = "fake"
            )

        q = package.Device(
            name = "system_bios(ven_0x5555_dev_0x1234)",
            version = "a00",
            displayname = "fake"
            )

        systemInventory = [p,q]
        r = repository.Repository(datafiles)
        updateSet = generateUpdateSet(r, systemInventory)

        self.assertEqual( updateSet.getUpdatePackageForDevice(p).name, "system_specific" )
        self.assertEqual( updateSet.getUpdatePackageForDevice(p).version, "a04" )

    def testGenerateUpdateSet_SystemSpecific3(self):
        # test the case where system specific update available that does
        # apply to current system
        import firmwaretools.repository as repository
        import firmwaretools.package as package

        p = package.Device(
            name = "system_specific",
            version = "a01",
            displayname = "fake"
            )

        q = package.Device(
            name = "system_bios(ven_0x5555_dev_0x4321)",
            version = "a00",
            displayname = "fake"
            )

        systemInventory = [p,q]
        r = repository.Repository(datafiles)
        updateSet = generateUpdateSet(r, systemInventory)

        self.assertEqual( updateSet.getUpdatePackageForDevice(p).name, "system_specific" )
        self.assertEqual( updateSet.getUpdatePackageForDevice(p).version, "a09" )


    def testGenerateUpdateSet_testRequires1(self):
        # test the case where system specific update available that doesn't
        # apply to current system
        import firmwaretools.repository as repository
        import firmwaretools.package as package

        p = package.Device(
            name = "test_requires",
            version = "a01",
            displayname = "fake"
            )

        systemInventory = [p,]
        r = repository.Repository(datafiles)
        updateSet = generateUpdateSet(r, systemInventory)

        self.failUnless( updateSet.getUpdatePackageForDevice(p) is None )

    def testGenerateUpdateSet_testRequires2(self):
        # test the case where system specific update available that does
        # apply to current system
        import firmwaretools.repository as repository
        import firmwaretools.package as package

        p = package.Device(
            name = "test_requires",
            version = "a01",
            displayname = "fake"
            )

        q = package.Device(
            name = "otherpackage",
            version = "a00",
            displayname = "fake"
            )

        r = package.Device(
            name = "foo",
            version = "43",
            displayname = "fake"
            )

        systemInventory = [p,q,r]
        r = repository.Repository(datafiles)
        updateSet = generateUpdateSet(r, systemInventory)

        self.assertEqual( updateSet.getUpdatePackageForDevice(p).name, "test_requires" )
        self.assertEqual( updateSet.getUpdatePackageForDevice(p).version, "a09" )

    def testInstallationOrder1(self):
        import firmwaretools.repository as repository
        import firmwaretools.package as package

        p = package.Device(
            name = "testorder1",
            version = "a01",
            displayname = "fake"
            )

        q = package.Device(
            name = "testorder2",
            version = "a01",
            displayname = "fake"
            )

        r = package.Device(
            name = "testorder3",
            version = "a01",
            displayname = "fake"
            )

        systemInventory = [r,p,q]
        r = repository.Repository(datafiles)

        installationOrder = ["testorder1", "testorder2", "testorder3" ]
        updateSet = generateUpdateSet(r, systemInventory)
        for pkg in updateSet.generateInstallationOrder():
            n = installationOrder[0]
            if len(installationOrder) > 1:
                installationOrder = installationOrder[1:]

            self.assertEqual( n, pkg.name )

    def testInstallationOrder2(self):
        import firmwaretools.repository as repository
        import firmwaretools.package as package

        p = package.Device(
            name = "testorder1",
            version = "a01",
            displayname = "fake"
            )

        q = package.Device(
            name = "testorder2",
            version = "a08",
            displayname = "fake"
            )

        r = package.Device(
            name = "testorder3",
            version = "a01",
            displayname = "fake"
            )

        systemInventory = [r,p,q]
        r = repository.Repository(datafiles)

        installationOrder = ["testorder1", "testorder3" ]
        updateSet = generateUpdateSet(r, systemInventory)
        for pkg in updateSet.generateInstallationOrder():
            n = installationOrder[0]
            if len(installationOrder) > 1:
                installationOrder = installationOrder[1:]

            self.assertEqual( n, pkg.name )


if __name__ == "__main__":
    import test.TestLib
    sys.exit(not test.TestLib.runTests( [TestCase] ))



# vim:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:tw=0
"""i18n abstraction

License: GPL
Author: Vladimir Bormotov <bor@vb.dn.ua>

$Id$
"""
# $RCSfile$
__version__ = "$Revision$"[11:-2]
__date__ = "$Date$"[7:-2]

try:
    import gettext
    import sys
    if sys.version_info[0] == 2:
        t = gettext.translation('yum')
        _ = t.ugettext
    else:
        gettext.bindtextdomain('yum', '/usr/share/locale')
        gettext.textdomain('yum')
        _ = gettext.gettext

except:
    def _(str):
        """pass given string as-is"""
        return str

if __name__ == '__main__':
    pass

# vim: set ts=4 et :



# vim:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:tw=0

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright (C) 2008 Dell Inc.
#  by Michael Brown <Michael_E_Brown@dell.com>

"""
Classes for subcommands of the yum command line interface.
"""

from firmwaretools.trace_decorator import decorate, traceLog, getLog
import firmwaretools.plugins as plugins

import ftcommands

plugin_type = (plugins.TYPE_CLI,)
requires_api_version = "2.0"

moduleLog = getLog()
moduleVerboseLog = getLog(prefix="verbose.")

def config_hook(conduit, *args, **kargs):
    conduit.getOptParser().addEarlyParse("--update")
    conduit.getOptParser().add_option(
        "--update", help="Update the system's firmware.",
        action="store_const", const="update", dest="mode")
    conduit.getBase().registerCommand(UpdateCommand())

class UpdateCommand(ftcommands.YumCommand):
    decorate(traceLog())
    def getModes(self):
        return ['update']

    decorate(traceLog())
    def addSubOptions(self, base, mode, cmdline, processedArgs):
        base.optparser.add_option("--rpm", action="store_true", dest="rpmMode", default=False, help="Used when running as part of an rpm \%post script.")
        base.optparser.add_option("--yes", "-y", action="store_const", const=0, dest="interactive", default=1, help="Default all answers to 'yes'.")
        base.optparser.add_option("--test", "-t", action="store_const", const=2, dest="interactive", help="Perform test but do not actually update.")
        base.optparser.add_option( "--show-unknown", help="Show unknown devices.", action="store_true", dest="show_unknown", default=False)
        base.optparser.add_option( "--storage-topdir", help="Override configured storage topdir.", action="store", dest="storage_topdir", default=None)

    decorate(traceLog())
    def doCheck(self, base, mode, cmdline, processedArgs):
        moduleVerboseLog.info("hello world from update module doCheck()")
        if base.opts.storage_topdir is not None:
            moduleLog.info("overriding storage topdir. Original: %s  New: %s" % (base.conf.storageTopdir, base.opts.storage_topdir))
            base.conf.storageTopdir = base.opts.storage_topdir

    decorate(traceLog())
    def doCommand(self, base, mode, cmdline, processedArgs):
        if base.opts.rpmMode:
            if base.conf.rpmMode != 'auto':
                print "Config does not specify automatic install during package install."
                print "Please run update_firmware manually to install updates."
                return [0, "Done"]

        base.updateFirmware(base.opts.show_unknown)
        return [0, "Done"]




# vim:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:tw=0

  #############################################################################
  #
  # Copyright (c) 2005 Dell Computer Corporation
  # Dual Licenced under GNU GPL and OSL
  #
  #############################################################################
"""
repository module
"""

from __future__ import generators

import os

import repository
import mockpackage
import sys
from firmwaretools.trace_decorator import decorate, traceLog, getLog
import firmwaretools.plugins as plugins

plugin_type = (plugins.TYPE_MOCK_INVENTORY, )
requires_api_version = "2.0"

moduleLog = getLog()
moduleVerboseLog = getLog(prefix="verbose.")

#
# DEBUG ONLY
#

# a null function that just eats args. Default callback
def nullFunc(*args, **kargs): pass

def config_hook(conduit, *args, **kargs):
    repository.Repository.iterPackages = iterPackages_DEBUG

decorate(traceLog())
def iterPackages_DEBUG(self, cb=(nullFunc, None)):
    # TODO: put this in a separate function
    yield mockpackage.MockRepositoryPackage(
        displayname="Baseboard Management Controller for Imaginary Server 1234",
        name="debug_system_bmc",
        version="0.9")
    yield mockpackage.MockRepositoryPackage(
        displayname="ReallyFast Network Controller",
        name="debug_pci_firmware_ven_crappy_dev_slow",
        version="1.1")
    yield mockpackage.MockRepositoryPackage(
        displayname="Pokey Modem -- Enhanced 1200baud",
        name="debug_pci_firmware_ven_0x0c64_dev_0xrocked",
        version="1.1")
    yield mockpackage.MockRepositoryPackage(
        displayname="Pokey Modem -- Enhanced 1200baud",
        name="debug_pci_firmware_ven_0x0c64_dev_0xrocked",
        version="1.9")
    yield mockpackage.MockRepositoryPackage(
        displayname="SafeData RAID Controller v2i",
        name="debug_pci_firmware_ven_corrupt_dev_yourdata",
        version="1.1")
    yield mockpackage.MockRepositoryPackage(
        displayname="SafeData RAID Controller v2i",
        name="debug_pci_firmware_ven_corrupt_dev_yourdata",
        version="2.9")
    yield mockpackage.MockRepositoryPackage(
        displayname="AdapFirm SloTek AHA-1501",
        name="debug_pci_firmware_ven_violates_dev_scsistandard",
        version="2.1")
    yield mockpackage.MockRepositoryPackage(
        displayname="AdapFirm SloTek AHA-1501",
        name="debug_pci_firmware_ven_violates_dev_scsistandard",
        version="2.5")
    yield mockpackage.MockRepositoryPackage(
        displayname="AdapFirm SloTek AHA-1501",
        name="debug_pci_firmware_ven_violates_dev_scsistandard",
        version="3.0")
    yield mockpackage.MockRepositoryPackage(
        displayname="PixelPusher 2000 Video Adapter",
        name="debug_pci_firmware_ven_draws_dev_polygons",
        version="4.0")
    yield mockpackage.MockRepositoryPackage(
        displayname="PixelPusher 2000 Video Adapter",
        name="debug_pci_firmware_ven_draws_dev_polygons",
        version="4.1")
    yield mockpackage.MockRepositoryPackage(
        displayname="PixelPusher 2000 Video Adapter",
        name="debug_pci_firmware_ven_draws_dev_polygons",
        version="4.1.1")
    yield mockpackage.MockRepositoryPackage(
        displayname="PixelPusher 2000 Video Adapter",
        name="debug_pci_firmware_ven_draws_dev_polygons",
        version="4.1.2")




# vim:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:tw=0

  #############################################################################
  #
  # Copyright (c) 2005 Dell Computer Corporation
  # Dual Licenced under GNU GPL and OSL
  #
  #############################################################################
"""module

some docs here eventually.
"""

from __future__ import generators

# import arranged alphabetically
import commands
import getopt
import glob
import os
import sys
import ConfigParser
import math
import zipfile
import re
import shutil
import signal
import time
import threading

from firmwaretools.trace_decorator import decorate, traceLog, getLog

def clearLine():
    return "\033[2K\033[0G"

def spinner(cycle=['/', '-', '\\', '|']):
    step = cycle[0]
    del cycle[0]
    cycle.append(step)
    # ESC codes for clear line and position cursor at horizontal pos 0
    return step

def pad(strn, pad_width=67):
    # truncate strn to pad_width so spinPrint does not scroll
    if len(strn) > pad_width:
        return strn[:pad_width] + ' ...'
    else:
        return strn

def spinPrint(strn, outFd=sys.stderr):
    outFd.write(clearLine())
    outFd.write("%s\t%s" % (spinner(), pad(strn)))
    outFd.flush()

def timedSpinPrint( strn, start ):
    now = time.time()
    # ESC codes for position cursor at horizontal pos 65
    spinPrint( strn + "\033[65G time: %2.2f" % (now - start) )


# helper class & functions for executeCommand()
# User should handle this if they specify a timeout
class commandTimeoutExpired(Exception): pass

# the problem with os.system() is that the command that is run gets any
# keyboard input and/or signals. This means that <CTRL>-C interrupts the
# sub-program instead of the python program. This helper function fixes that.
# It also allows us to set up a maximum timeout before all children are killed
decorate(traceLog())
def executeCommand(cmd, timeout=0):
    class alarmExc(Exception): pass
    def alarmhandler(signum,stackframe):
        raise alarmExc("timeout expired")

    pid = os.fork()
    if pid:
        #parent
        rpid = ret = 0
        oldhandler=signal.signal(signal.SIGALRM,alarmhandler)
        starttime = time.time()
        prevTimeout = signal.alarm(timeout)
        try:
            (rpid, ret) = os.waitpid(pid, 0)
            signal.alarm(0)
            signal.signal(signal.SIGALRM,oldhandler)
            if prevTimeout:
                passed = time.time() - starttime
                signal.alarm(int(math.ceil(prevTimeout - passed)))
        except alarmExc:
            try:
                os.kill(-pid, signal.SIGTERM)
                time.sleep(1)
                os.kill(-pid, signal.SIGKILL)
            except OSError: # errno=3  == no such process
                pass
            (rpid, ret) = os.waitpid(pid, 0)
            signal.signal(signal.SIGALRM,oldhandler)
            if prevTimeout:
                passed = time.time() - starttime
                signal.alarm(int(max(math.ceil(prevTimeout - passed), 1)))
            raise commandTimeoutExpired( "Specified timeout of %s seconds expired before command finished. Command was: %s"
                    % (timeout, cmd)
                    )
        except KeyboardInterrupt:
            signal.signal(signal.SIGALRM,oldhandler)
            try:
                os.kill(-pid, signal.SIGTERM)
                time.sleep(1)
                os.kill(-pid, signal.SIGKILL)
            except OSError: # errno=3  == no such process
                pass
            (rpid, ret) = os.waitpid(pid, 0)
            raise

        # mask and return just return value
        return (ret & 0xFF00) >> 8
    else:
        #child
        os.setpgrp()  # become process group leader so that we can kill all our children
        signal.signal(signal.SIGINT, signal.SIG_IGN) #ignore <CTRL>-C so parent gets it
        ret = os.system(cmd)
        os._exit( (ret & 0xFF00) >> 8 )

decorate(traceLog())
def copyFile( source, dest, ignoreException=0 ):
    try:
        shutil.copyfile(source, dest)
    except IOError:
        if not ignoreException:
            raise

# python 2.3 has a better version, but we have to run on python 2.2. :-(
decorate(traceLog())
def mktempdir( prefix="/tmp" ):
    status, output = commands.getstatusoutput("mktemp -d %s/tempdir-$$-$RANDOM-XXXXXX" % prefix)
    if status != 0:
        raise Exception("could not create secure temporary directory: %s" % output)
    return output

# generator function -- emulates the os.walk() generator in python 2.3 (mostly)
# ret = (path, dirs, files) foreach dir
decorate(traceLog())
def walkPath(topdir, direction=0):
    rawFiles = os.listdir(topdir)

    files=[f for f in rawFiles if os.path.isfile(os.path.join(topdir,f))]
    dirs =[f for f in rawFiles if os.path.isdir (os.path.join(topdir,f))]

    if direction == 0:
        yield (topdir, dirs, files)

    for d in dirs:
        if not os.path.islink(os.path.join(topdir,d)):
            for (newtopdir, newdirs, newfiles) in walkPath(os.path.join(topdir,d)):
                yield (newtopdir, newdirs, newfiles)

    if direction == 1:
        yield (topdir, dirs, files)


decorate(traceLog())
def runLongProcess(function, args=None, kargs=None, waitLoopFunction=None):
    # runs a function in a separate thread. Runs waitLoopFunction() while it
    # waits for the function to finish. Good for updating GUI, or other stuff
    thread = BackgroundWorker(function, args, kargs)
    while thread.running:
        if waitLoopFunction is not None:
            waitLoopFunction()

    # run waitLoopFunction one last time before exit.
    # gives status opportunity to update to 100%
    if waitLoopFunction is not None:
        waitLoopFunction()

    if thread.exception:
        getLog(prefix="verbose.").exception(thread.exception)        
        raise thread.exception
    return thread.returnCode

class BackgroundWorker(threading.Thread):
    def __init__ (self, function, args=None, kargs=None):
        threading.Thread.__init__(self)
        self.function = function
        self.args = args
        self.kargs = kargs

        self.exception = None
        self.returnCode = None
        self.running=1

        if self.args is None: self.args = []
        if self.kargs is None: self.kargs = {}

        self.start()

    decorate(traceLog())
    def run(self):
        try:
            self.returnCode = self.function(*self.args, **self.kargs)
        except (Exception,), e:
            self.exception = e

        self.running=0




# vim:tw=0:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:
"""
Yum plugin to automatically bootstrap packages.

The purpose of this plugin is to automatically always add firmware-tools
bootstrap to the yum transaction. Before this was done by an explicit added
step. The instructions used to say:

# yum install $(bootstrap_firmware)

Normal bootstrap_firmware output looks like this:

# bootstrap_firmware
pci_firmware(ven_0x10de_dev_0x00e1)
pci_firmware(ven_0x10de_dev_0x00e0_subven_0x10de_subdev_0x0c11)
pci_firmware(ven_0x10de_dev_0x00e4_subven_0x10de_subdev_0x0c11)
pci_firmware(ven_0x10de_dev_0x00e7_subven_0x10de_subdev_0x0c11)
... cut ...

The yum install command would effectively look up each pci_firmware(...) line
and look for RPM packages that had "Provides:" tags for the corresponding
firmware.

The purpose of this plugin is to remove that additional step. Now anytime the
user says "yum upgrade", they will automatically get the latest bootstrap
packages installed on their system.
"""

# TODO List:
#  -- command line option to disable
#  -- investigate using preresolve_hook w/ command line option to force
#  -- dont add firmware if user specifies pkgs in update cmd
#  -- way to not break CentOS 4

from yum.plugins import TYPE_CORE
import yum.Errors

version="1.5.10"
requires_api_version = '2.1'
plugin_type = (TYPE_CORE,)

import ConfigParser
import os
import sys
import time

global firmwaretools
import firmwaretools.clifuncs
import firmwaretools.trace_decorator as trace_decorator

if os.environ.get('DEBUG_BOOTSTRAP') == "1":
    # provide a made-up set of bootstrap packages rather than real one.
    # useful for testing.
    import firmwaretools.mockpackage

if os.environ.get('DEBUG'):
    # activate function tracing
    trace_decorator.debug["__main__"] = 9

def exclude_hook(conduit):
    start = time.time()
    print "exclude hook start: %s" % start
    ini = ConfigParser.ConfigParser()
    firmwaretools.clifuncs.getConfig(ini, firmwaretools.clifuncs.configLocations)
    for pkg in firmwaretools.clifuncs.runBootstrapInventory(ini):
        # see if it is a normal package name
        try:
            conduit._base.install(name=pkg.name)
            continue
        except yum.Errors.InstallError, e:
            pass

        # isnt a straight name, need to see if it is a dep
        try:
            mypkg = conduit._base.returnPackageByDep(pkg.name)
            conduit._base.install(mypkg)
        except yum.Errors.YumBaseError, e:
            pass
    print "exclude hook end: %s" % (time.time() - start)

## debugging stuff...
def debug_hook_calls(where, *args, **kargs):
    print "Where: %s" % where

import functools
for hook in (
    'config',
    'postconfig',
    'init',
    'predownload',
    'postdownload',
    'prereposetup',
    'postreposetup',
    'close',
    'clean',
    'pretrans',
    'posttrans',
    'exclude',
    'preresolve',
    'postresolve',
    ):
    hookname = "%s_hook" % hook
    setattr(sys.modules[__name__], hookname, functools.partial(debug_hook_calls, hookname))

# this decorates all functions in this module to provide tracing if it is enabled.
trace_decorator.decorateAllFunctions(sys.modules[__name__])



#!/usr/bin/env python
# VIM declarations
# vim:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:

#alphabetical order
import os
import unittest

def usage():
    print "wrong command line options." #need better help eventually

def parseOptions(*args):
    pass

def runTests( testCase ):
    testToRun = 'test'

    myTestSuite = unittest.TestSuite()
    for i in testCase:
        try:
            temp = unittest.makeSuite( i, testToRun )
            myTestSuite.addTest(temp)
        except ValueError:
            pass

    runner = unittest.TextTestRunner( verbosity=3 )
    retval = runner.run(myTestSuite)

    return retval.wasSuccessful()

    #END main( testCase ):

main = runTests

def areFilesDifferent( orig, copy ):
    ret = os.system( "diff -q %s %s >/dev/null 2>&1" % (orig, copy) ) >> 8
    if ret == 0:
        return 0
    if ret == 1:
        return 1
    raise "something bad happened"

def copyFile( source, dest ):
    fileOrig = open( source, "r" )
    fileCopy = open( dest, "w" )

    line = fileOrig.read( 512 )
    while line != "":
        fileCopy.write(line)
        line = fileOrig.read( 512 )

    fileOrig.close()
    fileCopy.close()

if __name__ == "__main__":
    print "This file is not executable."
